%% Name:		fldcmp.m
%% Description:	Field comparison for Matlab structures
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as 
%%				published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed
%%				in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
%%				FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU
%%				General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

function [names,fields1,fields2]=fldcmp(struct1,struct2,relation)

	%% Syntax:
	%%
	%%		[names,fields1,fields2]=fldcmp(struct1,struct2,'relation')
	%%
	%% Description:
	%%
	%%		[names,fields1,fields2]=fldcmp(struct1,struct2,'relation') compares the fields of the two structures struct1 and struct2 using the 
	%%		relational operator specified by 'relation'.
	%%
	%%			Arguments:
	%%				structN:	Structures to be compared. The structures must have the same number and name of fields.
	%%
	%%				'relation':	String describing the relational operator to be used for field comparison. Supported operators are '==' for
	%%							equality and '~=' for inequality
	%%
	%%			Returns:
	%%				names:		Cell array of matching fieldnames
	%%				fieldsN:	Cell array of matching fields
	%%

	%% Assert relational operator argument
	switch(relation)
		case {'==','eq'}
			flag_eq=true;
		case {'~=','ne'}
			flag_eq=false;
		otherwise
			throw(MException('varargin2pairs:BadRel','Unknown "relation" argument. Expected "==" or "~=". Received %s',relation));
	end
	
	%% Assert field names
	struct1=orderfields(struct1);
	struct2=orderfields(struct2);
	names.struct1=fieldnames(struct1);
	names.struct2=fieldnames(struct2);
	if(length(names.struct1)~=length(names.struct2))
		throw(MException('varargin2pairs:MismatchedFieldLengths','Number of fields do not match. Number of fields in struct1: %d. Number of fields in struct2: %d',length(names.struct1),length(names.struct2)));
	end
	if(~all(strcmp(names.struct1,names.struct2)))
		throw(MException('varargin2pairs:MismatchedFieldNames','Field names do not match'));
	end
	
	%% Compare field values
	fields.struct1=struct2cell(struct1);					% Must convert to cell array as structfun only supports one structure at a time
	fields.struct2=struct2cell(struct2);
	if(flag_eq)
		fun=@(x,y) isequal(x,y);
	else
		fun=@(x,y) ~isequal(x,y);
	end
	mask=cellfun(fun,fields.struct1,fields.struct2,'UniformOutput',true);
	if(any(mask))
		names=names.struct1(mask);
		fields1=fields.struct1(mask);
		fields2=fields.struct2(mask);
	else
		names=[];
		fields1=[];
		fields2=[];
	end

end