%% Name:		ini2struct.m
%% Description:	Parse the contents of an INI formatted string into a Matlab structure
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as 
%%				published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed
%%				in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
%%				FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU
%%				General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

function ini_struct=ini2struct(ini_str)

	%% Syntax:
	%%
	%%		ini_struct=ini2struct('ini_str')
	%%
	%% Description:
	%%
	%%		ini_struct=ini2struct('ini_str') parses the contents of the INI formatted string 'ini_str' into the structure ini_struct
	%%
	%%			Arguments:
	%%				ini_str:	String to be parsed. The name of every section should be enclosed in square brackets '[]'. Tokens are assumed
	%%							to be separated from their corresponding values by an equals character '='. Token-value pairs are separated
	%%							from each other by commas ','. Values can be enclosed in double quotes '"' but this is optional. If quoted,
	%%							values can contain commas (e.g. [Section],Tok="Val1,Val2",). Section and token names are case sensitive.
	%%
	%%			Returns:
	%%				ini_struct:	Structure containing the token-value pairs. The structure has the format ini_struct.'section'.'token' with all
	%%							values stored as strings. e.g. The number of exposures ('NrExposure' in a section titled 'Acquisition') can be
	%%							found using ini_struct.Acquisition.NrExposure
	%%

	
	%% Remove newlines
	ini_str=strrep(ini_str,sprintf('\r\n'),'');
	
	%% Substitute delimited commas
	SUB=26;
	ini_str=regexprep(ini_str,'(?<=="[^"]*),',char(SUB));
	
	%% Substitute delimited square brackets
	STX=2;ETX=3;
	ini_str=regexprep(ini_str,{'(?<=="[^"]*)[','(?<=="[^"]*)]'},{char(STX),char(ETX)});
	
	%% Parse into sections
	secs=regexp(ini_str,'[','split');
	secs=secs(2:end);
	
	%% Parse into key-value pairs
	[sec_names,sec_structs]=cellfun(@ini2struct_ParseSec,secs,'UniformOutput',false);
	
	%% Build structure
	ini_struct=cell2struct(sec_structs,sec_names,2);
	
end

function [sec_name,sec_struct]=ini2struct_ParseSec(sec)

	%% Parse section name
	sec_name=regexp(sec,'^(.*)]','tokens');
	sec_name=regexprep(sec_name{1}{1},'[^A-Za-z0-9]','');								% Remove non-alphanumerics for valid field names (warn?)
	if(~isempty(regexp(sec_name,'^\d+[A-Za-z]*','once')))								% Handle numerical field name start
		warning('ini2struct:NumSecStart','Non-alphabetical value found at start of section name (%s). Prepended "d" to field name',sec_name);
		sec_name=['d',sec_name];
	end
	
	%% Parse key-value pairs
	kvp=regexp(regexp(sec,',','split'),'=','split');
	kvp=vertcat(kvp{2:end});
	kvp(:,1)=regexprep(kvp(:,1),'[^A-Za-z0-9]','');										% Remove non-alphanumerics for valid field names (warn?)
	kvp(:,1)=regexprep(kvp(:,1),'(^\d+[A-Za-z]*)','d$1');								% Handle numerical field name start (prepend 'd')
	kvp(:,2)=strrep(kvp(:,2),'"','');													% Remove quotation marks
	SUB=26;STX=2;ETX=3;
	kvp(:,2)=regexprep(kvp(:,2),{char(SUB),char(STX),char(ETX)},{',','[',']'});			% Replace delimited commas and square brackets
	
	%% Build section structure
	sec_struct=cell2struct(kvp(:,2),kvp(:,1),1);

end