%% Name:		struct2ini.m
%% Description:	Reformat the contents of a Matlab structure into an INI formatted string
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as 
%%				published by the Free Software Foundation, either version 3 of the License, or any later version. This program is distributed
%%				in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
%%				FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU
%%				General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

function ini_str=struct2ini(ini_struct)
	
	%% Syntax:
	%%
	%%		ini_str=struct2ini(ini_struct)
	%%
	%% Description:
	%%
	%%		ini_str=struct2ini(ini_struct) reformats the contents of the structure ini_struct into the INI formatted string ini_str.
	%%
	%%			Arguments:
	%%				ini_struct:	Structure containing token-value pairs. The structure has the format ini_struct.'section'.'token' with all
	%%							values stored as strings. e.g. The number of exposures ('NrExposure' in a section titled 'Acquisition') can be
	%%							found using ini_struct.Acquisition.NrExposure
	%%
	%%			Returns:
	%%				ini_str:	INI formatted string. The name of every section is enclosed in square brackets '[]'. Tokens are separated from
	%%							their corresponding values by an equals character '='. Token-value pairs are separated from each other by commas
	%%							','. Values can be enclosed in double quotes '"' but this is optional. If quoted, values can contain commas
	%%							(e.g. [Section],Tok="Val1,Val2",). Section and token names are case sensitive.
	%%


	%% Get section names
	sec_names=cellfun(@(sec_name) ['[',sec_name,'],'] ,fieldnames(ini_struct),'UniformOutput',false);	% Could handle reconversion of numeric fieldnames here

	%% Get section contents
	sec_cell=struct2cell(ini_struct);
	
	%% Convert section contents to strings
	kvp_cell=cellfun(@struct2ini_KVPStr,sec_cell,'UniformOutput',false);
	
	%% Concatenate section names and content strings
	ini_str=cellfun(@(names,kvps) [names,kvps,sprintf('\r\n')],sec_names,kvp_cell,'UniformOutput',false);
	ini_str=cell2mat(ini_str');
	ini_str=ini_str(1:end-2);														% Remove final excess newline
	
end

function kvp=struct2ini_KVPStr(sec_struct)

	%% Get keys
	keys=fieldnames(sec_struct);
	
	%% Get values
	values=struct2cell(sec_struct);
	values=cellfun(@(vals) ['"',vals,'"'] ,values,'UniformOutput',false);			% Quote all values
	
	%% Concatenate keys and values
	kvp=cellfun(@(k,v) [k,'=',v,','],keys,values,'UniformOutput',false);
	kvp=cell2mat(kvp');
	kvp=kvp(1:end-1);																% Remove final excess comma

end
