%% Name:		trplCurCor.m
%% Description:	Apply curvature correction to data from Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function [data,parab_coeffs] = trplCurCor(data,wl,t,parab_pts)

	%% Syntax:
	%%
	%%		data = trplCurCor(data,wl,t,parab_pts)
	%%
	%% Description:
	%%
	%%		data = trplCurCor(data,wl,t,parab_pts) corrects Chameleon TRPL data for curvature distortions arising from photoelectron
	%%		collimation. Correction is performed by temporal interpolation along a parabolic spectral fit determined by parab_pts argument.
	%%
	%%			Arguments:
	%%				data:		2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				wl:			Column vector defining the horizontal wavelength axis with wl(i) corresponding to data(:,i).
	%%				t:			Column vector defining the vertical time axis with t(i) corresponding to data(i,:).
	%%				parab_pts:	Column vector of three points for which to fit a correction parabola. The points are grouped in rows such
	%%							that parab_pts(i,1) defines a wavelength value and parab_pts(i,2) defines the corresponding time value.
	%%
	%%			Returns:
	%%				data:		2D array of curvature corrected TRPL intensities in the same format as returned by trplLoad.
	

	%% Check and rarrange parabola argument
	if (~(isequal(size(parab_pts),[3,2])))										% Check dimensions
		throw(MException('trpl:CurCor:ParabDimBad','The dimensions of parab_pts argument are incorrect. Expected a 3 by 2 array, received a %d by %d array',size(parab_pts,1),size(parab_pts,2)));
	end
	if (min(parab_pts(:,1))<min(wl) || max(parab_pts(:,1))>max(wl))				% Check wavelengths are within spectral range
		throw(MException('trpl:CurCor:ParabWlOutOfRange','One or more wavelength values in argument parab_pts are out of spectral range. Expected values between %g and %g, received values between %g and %g',min(wl),max(wl),min(parab_pts(:,1)),max(parab_pts(:,1))));
	end
	if (min(parab_pts(:,2))<min(t) || max(parab_pts(:,2))>max(t))				% Check times are within temporal range
		throw(MException('trpl:CurCor:ParabTOutOfRange','One or more time values in argument parab_pts are out of temporal range. Expected values between %g and %g, received values between %g and %g',min(t),max(t),min(parab_pts(:,2)),max(parab_pts(:,2))));
	end
	
	%% Calculate parabola coefficients
	parab_coeffs=([(parab_pts(:,1).^2),parab_pts(:,1),[1;1;1]])\(parab_pts(:,2));
	parab_coeffs(3)=(parab_coeffs(2).^2)/(4*parab_coeffs(1));					% Remove parabola offset to avoid translating data
	
	%% Generate parabola at TRPL wavelengths
	parab=([(wl.^2),wl,repmat(1,length(wl),1)])*parab_coeffs;
	
	%% Generate 2D interpolation mesh grids
	[mesh_wl,mesh_t]=meshgrid(wl',t);
	mesh_parab=mesh_t+repmat(parab',size(mesh_t,1),1);
	
	%% 2D interpolation of data at t-t_parabola(wl)
	switch(class(data))
		case {'uint8','int16','int32','single','double'}
			data=cast(interp2(mesh_wl,mesh_t,single(data),mesh_wl,mesh_parab,'linear'),class(data));
		otherwise
			throw(MException('trpl:CurCor:DataType','Unsupported data datatype. Expected uint8, int16, int32, single or double. Received: %s',class(data)));
	end

end