%% Name:		trplGetKin.m
%% Description:	Get kinetics (i.e. PL(time)) from Time Resolved PL data taken in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function [kins,varargout] = trplGetKin(data,wl,wls)

	%% Syntax:
	%%
	%%		kins = trplGetKin(data,wl,wls)
	%%		[kins,ints] = trplGetKin(data,wl,wls)
	%%
	%% Description:
	%%
	%%		kins = trplGetKin(data,wl,wls) gets kinetics (columns) from Chameleon TRPL data at specified wavelengths.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				wl:		Column vector defining the horizontal wavelength axis with wl(i) corresponding to data(:,i).
	%%				wls:	Column vector of wavelengths at which to get kinetics
	%%
	%%			Returns:
	%%				kins:	Array of horizontally concatenated kinetics. kins(:,i) is the kinetic at wavelength wls(i)
	%%
	%%
	%%		[kins,inds] = trplGetKin(data,wl,wls) get kinetics from Chameleon TRPL data integrated over specified wavelength intervals.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				wl:		Column vector defining the horizontal wavelength axis with wl(i) corresponding to data(:,i).
	%%				wls:	n*2 array of wavelength intervals over which to get integrated kinetics. Intervals must be contained within rows
	%%						such that wls(:,1) are the interval start values and wls(:,2) are the interval end values.
	%%			Returns:
	%%				kins:	Array of horizontally concatenated integrated kinetics.
	%%				inds:	Integration interval indices. kins(:,i) is the spectrum integrated over interval wl(inds(:,i)). In most cases
	%%						this is equal to wls, however this canot be guaranteed hence why inds is supplied. N.B. Unlike in wls, the
	%%						intervals in inds are contained in columns! This should make it easier to match the intervals with the returned
	%%						spectra.
	
	
 	%% Define integration inline function for arrayfun
	function trplGetKin_Int(ind_start,ind_end)
		%% Get integrated kinetics, vertically concatenate to wavelength indices, and horizontally concatenate to previous spectra
		%%		- This modifies 'kins' directly
		%%		- Wavelength indices are concatenated as order of arrayfun execution not specifically defined
		%%		- Index order must be correct (i.e. low, high) else trapz returns 0
		%%		- Wavelength axis must be increasing with index else trapz returns negative intergal
 		
		if(ind_start==ind_end)
			%% Same interval start and end values
			kins=[kins,[ind_start;ind_end;data(:,ind_start)]];
		else
			%% Different interval start and end values
			if (wl(ind_start)<wl(ind_end))
				kins=[kins,[ind_start;ind_end;trapz(wl(ind_start:ind_end),double(data(:,ind_start:ind_end)),2)]];
			else
				kins=[kins,[ind_start;ind_end;-trapz(wl(ind_start:ind_end),double(data(:,ind_start:ind_end)),2)]];
			end
		end
	end
	
	%% Select single kinetics or integrated kinetics
	switch size(wls,2)
		
		%% Single kinetics
		case 1
			
			%% Check wavelengths are within spectral range
			if (min(wls)<min(wl) || max(wls)>max(wl))
				throw(MException('trpl:GetKin:WLOutOfRange','One or more wls values are out of spectral range. Expected values between %g and %g, received values between %g and %g',min(wl),max(wl),min(wls),max(wls)));
			end
			
			%% Find wavelength indices
			wls_inds=round(interp1(wl,(1:length(wl))',wls));									% This gives indices closest to values of wls
			
			%% Extract kinetics at specified wavelengths
			kins=double(data(:,wls_inds));
			
		%% Integrated spectra
		case 2
			
			%% Check wavelengths are within spectral range
			if (min(min(wls))<min(wl) || max(max(wls))>max(wl))
				throw(MException('trpl:GetKin:WLOutOfRange','One or more wls values are out of spectral range. Expected values between %g and %g, received values between %g and %g',min(wl),max(wl),min(min(wls)),max(max(wls))));
			end
			
			%% Find wavelength indices
			wls_inds=round(interp1(wl,(1:length(wl))',wls));									% This gives indices closest to values of wls
			
			%% Sort wavelength indices for each interval 
			shift_inds=find(wls_inds(:,1)>wls_inds(:,2));										% Find rows which are in wrong order
			if (~(isempty(shift_inds)))
				wls_inds(shift_inds,:)=circshift(wls_inds(shift_inds,:),[0,1]);					% Swap columns in offending rows
			end
			
			%% Initialise return variable for trplGetKin_Int
			kins=[];
			
			%% Extract kinetics within specified wavelength intervals
			%%		N.B. No return variable specified as trplGetKin_Int modifies kins directly
			arrayfun(@trplGetKin_Int,wls_inds(:,1),wls_inds(:,2));
			
			%% Detach wavelength indices and place in varargout
			kins=double(kins);
			varargout(1)={kins(1:2,:)};														% Should this be transposed to match input, or leave as is to match output?
			kins(1:2,:)=[];
			
		%% Error - wls is not n*2 array
		otherwise
			throw(MException('trpl:GetKin:BadWlsDim','Dimension of wls argument incorrect. Expected 1 or 2 column argument, received %d column argument',size(wls,2)));
	end

end