%% Name:		trplGetSpec.m
%% Description:	Get spectra (i.e. PL(lambda)) from Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function [specs,varargout] = trplGetSpec(data,t,times)

	%% Syntax:
	%%
	%%		specs = trplGetSpec(data,t,times)
	%%		[specs,inds] = trplGetSpec(data,t,times)
	%%
	%% Description:
	%%
	%%		specs = trplGetSpec(data,t,times) gets spectra (rows) from Chameleon TRPL data at specified times.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				t:		Column vector defining the vertical time axis with t(i) corresponding to data(i,:).
	%%				times:	Column vector of times at which to get spectra.
	%%
	%%			Returns:
	%%				specs:	Array of vertically concatenated spectra. specs(i,:) is the spectrum at time times(i).
	%%
	%%
	%%		[specs,inds] = trplGetSpec(data,t,times) get spectra from Chameleon TRPL data integrated over specified time intervals.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				t:		Column vector defining the vertical time axis with t(i) corresponding to data(i,:).
	%%				times:	n*2 array of time intervals over which to get integrated spectra. Intervals must be contained within rows
	%%						such that times(:,1) are the interval start values and times(:,2) are the interval end values.
	%%			Returns:
	%%				specs:	Array of vertically concatenated integrated spectra.
	%%				inds:	Integration interval indices. specs(i,:) is the spectrum integrated over interval t(inds(i,:)). In most
	%%						cases this is equal to times, however this canot be guaranteed hence why inds is supplied.
	
	
 	%% Define integration inline function for arrayfun
	function trplGetSpec_Int(ind_start,ind_end)
		%% Get integrated spectra, horizontally concatenate to time indices, and vertically concatenate to previous spectra
			%%		- This modifies 'specs' directly
			%%		- Time indices are concatenated as order of arrayfun execution not specifically defined
			%%		- Index order must be correct (i.e. low, high) else trapz returns 0
			%%		- Time axis must be increasing with index else trapz returns negative intergal
		if(ind_start==ind_end)
			%% Same interval start and end values
			specs=[specs;[ind_start,ind_end,data(ind_start,:)]];
		else
			%% Different interval start and end values
			if (t(ind_start)<t(ind_end))
				specs=[specs;[ind_start,ind_end,trapz(t(ind_start:ind_end),double(data(ind_start:ind_end,:)),1)]];
			else
				specs=[specs;[ind_start,ind_end,-trapz(t(ind_start:ind_end),double(data(ind_start:ind_end,:)),1)]];
			end
		end
	end
	
	%% Select single spectra or integrated spectra
	switch size(times,2)
		
		%% Single spectra
		case 1
			
			%% Check times are within temporal range
			if (min(times)<min(t) || max(times)>max(t))
				throw(MException('trpl:GetSpec:TimesOutOfRange','One or more times values are out of temporal range. Expected values between %g and %g, received values between %g and %g',min(t),max(t),min(times),max(times)));
			end
			
			%% Find time indices
			times_inds=round(interp1(t,(1:length(t))',times));									% This gives indices closest to values of times
			
			%% Extract spectra at specified times
			specs=double(data(times_inds,:));
			
		%% Integrated spectra
		case 2
			
			%% Check times are within temporal range
			if (min(min(times))<min(t) || max(max(times))>max(t))
				throw(MException('trpl:GetSpec:TimesOutOfRange','One or more times values are out of temporal range. Expected values between %g and %g, received values between %g and %g',min(t),max(t),min(min(times)),max(max(times))));
			end
			
			%% Find time indices
			times_inds=round(interp1(t,(1:length(t))',times));									% This gives indices closest to values of times
			
			%% Sort time indices for each interval 
			shift_inds=find(times_inds(:,1)>times_inds(:,2));									% Find rows which are in wrong order
			if (~(isempty(shift_inds)))
				times_inds(shift_inds,:)=circshift(times_inds(shift_inds,:),[0,1]);				% Swap columns in offending rows
			end
			
			%% Initialise return variable for trplGetSpec_Int
			specs=[];
			
			%% Extract spectra within specified time intervals
			%%		N.B. No return variable specified as trplGetSpec_Int modifies specs directly
			arrayfun(@trplGetSpec_Int,times_inds(:,1),times_inds(:,2));
			
			%% Detach time indices and place in varargout
			specs=double(specs);
			varargout(1)={specs(:,1:2)};
			specs(:,1:2)=[];
			
		%% Error - times is not n*2 array
		otherwise
			throw(MException('trpl:GetSpec:BadTimesDim','Dimension of times argument incorrect. Expected 1 or 2 column argument, received %d column argument',size(times,2)));
	end
	
end