%% Name:		trplKinTool.m
%% Description:	Tool for manipulating kinetics of Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function outstruct=trplKinTool(file_name,wls,varargin)
	
	%% Syntax:
	%%
	%%		outstruct = trplKinTool('filename',wls)
	%%		outstruct = trplKinTool('filename',wls,'OptionName',OptionValue)
	%%
	%% Description:
	%%
	%%		outstruct = trplKinTool('filename',wls) opens the Chameleon TRPL data contained in 'filename' and performs operations on the
	%%		integrated kinetics over the wavelength interval wls. The default operation is to set the time of peak photoluminescence to 0
	%%		and subtract the mean of the pre-rise time signal.
	%%
	%%			Arguments:
	%%				'filename':	A single filename or a cell array of multiple filenames, as either absolute or relative path strings. If
	%%							'filename' is an empty array (i.e. []), a dialog is displayed to choose files from the filesystem.
	%%				wls:		Wavelength intervals over which to get integrated kinetics. A single set of intervals (i.e. one n*2 array) or 
	%%							multiple sets of intervals (i.e. a cell array with m nm*2 arrays) are accepted. If a single interval is given,
	%%							it is used for all files in the 'filename' argument. If multiple sets of intervals are given, the interval set
	%%							wls{i} will be used for filename{i}, and thus the dimensions of wls and 'filename' must agree.
	%%
	%%			Returns:
	%%				outstruct:	A structure containing the modified kintics. outstruct will contain a structure for each file in 'filenames'
	%%							with the sub-fields t and pl containing the time axis and integrated kinetic respectively.
	%%
	%%
	%%		outstruct = trplKinTool('filename',wls,'OptionName',OptionValue) sets named option 'OptionName' to the value OptionValue. Options
	%%		modify the operations which trplKinTool performs to the kinetics. Valid option names/values are:
	%%
	%%			'OptionName'		OptionValue	{default}			Description
	%%
	%%			'T0'				{true} | false					Set the time of peak photoluminescence to 0
	%%			'Subtract'			{true} | false					Substract the mean of the pre-rise time signal
	%%			'Crop'				true | {false}					Remove points before t=0
	%%			'Confirm'			{true} | false					Require user confirmation of peak PL time and subtraction values
	%%			'Normalize'			{true} | false					Normliaze the kinetics to unity at t=0
	%%			'Scaling'			numeric array {ones}			Scaling coefficients for kinetics such that scaling(i) acts on filename{i}
	%%			'ToWorkspace'		true | {false}					Export the modified kinetics to the base workspace as arrays
	%%			'ToCFTool'			true | {false}					Export the modified kinetics to the MATLAB curve-fitting tool
	%%			'MaxFraction'		numeric {1}						Fractional value of the maximum intensity at to set as t0
	%%			'StDevFraction'		numeric {auto}					Fractional value of the input kinetic standard-deviation for use in 
	%%																adaptive local regression filtering for finding rising-edge
	%%			'Sigma'				numeric {3}						Gaussian standard-deviation for identification of background
	%%

	%% Parse varargin
	[varg.names,varg.values]=varargin2pairs(varargin);
	opts=struct('stdevfraction',[],'maxfraction',1,'sigma',3,'scaling',[],'toworkspace',false,'tocftool',false,'crop',false,'confirm',true,'normalize',true,'t0',true,'subtract',true);		% Default options
	for ind_opts=1:(length(varg.names))
		switch(lower(varg.names{ind_opts}))
			case 'stdevfraction'
				if(~setOptionFloat('stdevfraction'))
					warning('trpl:KinTool:OptStDev','Option "StDevFraction" value not a scalar fraction. Expected value in interval ]0,1]. Received: %g. Continuing with default option value (%g)',varg.values{ind_opts},opts.stdevfraction);
				end
			case 'maxfraction'
				if(~setOptionFloat('maxfraction'))
					warning('trpl:KinTool:OptMaxFrac','Option "MaxFraction" value not a scalar fraction. Expected value in interval ]0,1]. Received: %g. Continuing with default option value (%g)',varg.values{ind_opts},opts.maxfraction);
				end
			case 'sigma'
				if(~setOptionFloat('sigma'))
					warning('trpl:T0Find:OptSigma','Option "Sigma" value not a positive scalar. Expected value in interval ]0,10] Received: %g. Continuing with default option value (%g)',varg.values{ind_opts},opts.sigma);
				end
			case 'scaling'
				if(~setOptionNum('scaling'))
					warning('trpl:KinTool:OptScaling','Option "Scaling" value not a numeric array. Continuing with default option value');
				end
			case 'toworkspace'
				if(~setOptionLogical('toworkspace'))
					warning('trpl:KinTool:OptWksp','Option "ToWorkspace" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.toworkspace);
				end
			case 'tocftool'
				if(~setOptionLogical('tocftool'))
					warning('trpl:KinTool:OptCFTool','Option "ToCFTool" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.tocftool);
				end
			case 'crop'
				if(~setOptionLogical('crop'))
					warning('trpl:KinTool:OptCrop','Option "Crop" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.crop);
				end
			case 'confirm'
				if(~setOptionLogical('confirm'))
					warning('trpl:KinTool:OptConf','Option "Confirm" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.confirm);
				end
			case {'normalize','normalise'}
				if(~setOptionLogical('normalize'))
					warning('trpl:KinTool:OptNorm','Option "Normalize" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.normalize);
				end
			case 't0'
				if(~setOptionLogical('t0'))
					warning('trpl:KinTool:OptT0','Option "T0" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.t0);
				end
			case 'subtract'
				if(~setOptionLogical('subtract'))
					warning('trpl:KinTool:OptSub','Option "Subtract" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.subtract);
				end
			otherwise
				throw(MException('trpl:KinTool:BadOpt','Option not recognised: %s',varg.names{ind_opts}));
		end
	end
		
	%% Get filenames (if none given)
	file_dir=[];
	if(isempty(file_name))
		exts={'*.img','ITEX image (*.img)';'*.dac','Calibrated ASCII image (*.dac)';'*.*','All Files (*.*)'};
		[file_name,file_dir]=uigetfile(exts,'Multiselect','on');
		if(isscalar(file_name) && file_name==0)
			return
		end
	end

	%% Encapsulate single path
	if(~iscell(file_name))
		file_name={file_name};
	end
	no_files=length(file_name);

	%% Replicate and encapsulate single wavelength interval set
	if(~iscell(wls))
		c_wls=cell(1,no_files);
		c_wls(:)={wls};
		wls=c_wls;
		clear('c_wls');
	end

	%% Pre-loop checks
	if(length(wls)~=no_files)
		throw(MException('trpl:KinTool:IntervalDims','Inconsistent wavelength intervals dimensions. Number of wavelength intervals must equal number of files. Number of files: %d. Number of wavelength intervals: %d',no_files,length(wls)));
	end
	if(~isempty(opts.scaling) && length(opts.scaling)~=no_files)
		throw(MException('trpl:KinTool:ScalingDims','Inconsistent scaling dimensions. Number of scaling coefficients must equal number of files. Number of files: %d. Number of scaling coefficients intervals: %d',no_files,length(opts.scaling)));
	end

	%% Loop through files
	for ind_file=1:length(file_name)

		%% Build file path
		path=[file_dir,file_name{ind_file}];
		[dir,name]=fileparts(path);

		%% Load files
		[data,wl,t]=trplLoad(path);
		
		%% Get kinetics
		kins=trplGetKin(data,wl,wls{ind_file});
		
		%% Normalise to interval width and scaling coefficient
		scale=abs(diff(wls{ind_file}')');
		if(~isempty(opts.scaling))
			scale=scale/(opts.scaling(ind_file));
		end
		kins=bsxfun(@rdivide,kins,scale');
		
		%% Loop through intervals
		for ind_int=1:size(wls{ind_file},1);
			
			%% Define current kinetic
			kin=kins(:,ind_int);

			%% Handle confirm option
			if(opts.confirm)

				%% Create t0 confirmation figure and axes
				h_conf.fig=figure('NumberTitle','off','Name','trplKinTool - Confirmation','color',get(0,'defaultUicontrolBackgroundColor'),'ToolBar','figure','Position',[100,50,1000,600],'WindowButtonDownFcn',@trplKinTool_fig_WindowButtonDown_Callback,'WindowButtonUpFcn',@trplKinTool_fig_WindowButtonUp_Callback);
				pos_conf.axout=[0,0,0.85,1];
				h_conf.ax=axes('OuterPosition',pos_conf.axout,'FontSize',8,'Tag','axConf');																			%#ok<LAXES>
				pos_conf.ax=get(h_conf.ax,'Position');

				%% Determine and plot filtered kinetics
				warning('off','trpl:T0Find:OptStDev');															% Disables warning on empty sdtdevfraction
				[t0.val,t0.ind,sub.ind]=trplT0Find(kin,t,'draw',true,'maxfraction',opts.maxfraction,'stdevfraction',opts.stdevfraction,'sigma',opts.sigma);
				warning('on','trpl:T0Find:OptStDev');
				sub.val=mean(kin(sub.ind));
				if(~isfinite(sub.val))
					sub.val=0;
				end

				%% Cosmetic changes
				Ylims=get(h_conf.ax,'YLim');
				set(h_conf.ax,'YLim',Ylims+(diff(Ylims)*0.05)*[-1,1],'XLim',[0,t0.val*2]);
				title(h_conf.ax,name,'Interpreter','none','FontWeight','bold');
				delete(findobj(get(h_conf.ax,'Children'),'flat','Tag','T0Vert'));
				delete(findobj(get(h_conf.ax,'Children'),'flat','Tag','T0Horiz'));
				delete(findobj(get(h_conf.ax,'Children'),'flat','Tag','T2SigVert'));

				%% Set up uicontrols

					%% Panel parameters
					pos_conf.pans.margins=struct('hor',0.05,'vert',0.05);
					pos_conf.pans.dims=struct('width',1-pos_conf.ax(1)-pos_conf.ax(3)-(2*pos_conf.pans.margins.hor),'height',0.12);

					%% T0 panel
					pos_conf.panT0=[pos_conf.ax(1)+pos_conf.ax(3)+pos_conf.pans.margins.hor,1-pos_conf.pans.dims.height-pos_conf.pans.margins.vert,pos_conf.pans.dims.width,pos_conf.pans.dims.height];
					h_conf.panT0=uipanel('Title','t0','Units','normalized','Position',pos_conf.panT0);

					%% Get norm:char ratio
					pos.norm=get(h_conf.panT0,'Position');
					set(h_conf.panT0,'Units','characters');
					pos.char=get(h_conf.panT0,'Position');
					set(h_conf.panT0,'Units','normalized');
					pos.char2norm=pos.norm(4)/pos.char(4);

					%% Subtraction panel
					pos_conf.panSub=[pos_conf.panT0(1),pos_conf.panT0(2)-pos_conf.panT0(4)-pos_conf.pans.margins.vert,pos_conf.panT0(3:4)];
					h_conf.panSub=uipanel('Title','Subtraction','Units','normalized','Position',pos_conf.panSub);

					%% Etxt parameters
					pos_conf.etxts.margins=struct('hor',0.1,'vert',0.1);
					pos_conf.etxts.dims=struct('width',1-(2*pos_conf.etxts.margins.hor),'height',(2.25*pos.char2norm)/pos_conf.pans.dims.height);

					%% T0 etxt
					pos_conf.etxtT0=[pos_conf.etxts.margins.hor,(1-pos_conf.etxts.dims.height)/2,pos_conf.etxts.dims.width,pos_conf.etxts.dims.height];
					h_conf.etxtT0=uicontrol(h_conf.fig,'Style','edit','Parent',h_conf.panT0,'Tag','etxtT0','String',num2str(t0.val),'Units','normalized','Position',pos_conf.etxtT0,'BackgroundColor','white','Callback',@trplKinTool_etxt_Callback);
					set(findjobj(h_conf.etxtT0),'FocusGainedCallback',@trplKinTool_etxt_FocusGainedCallback);

					%% Subtraction etxt
					pos_conf.etxtSub=[pos_conf.etxts.margins.hor,(1-pos_conf.etxts.dims.height)/2,pos_conf.etxts.dims.width,pos_conf.etxts.dims.height];
					h_conf.etxtSub=uicontrol(h_conf.fig,'Style','edit','Parent',h_conf.panSub,'Tag','etxtSub','String',num2str(sub.val),'Units','normalized','Position',pos_conf.etxtSub,'BackgroundColor','white','Callback',@trplKinTool_etxt_Callback);
					set(findjobj(h_conf.etxtSub),'FocusGainedCallback',@trplKinTool_etxt_FocusGainedCallback);

					%% PButton parameters
					pos_conf.etxts.dims=struct('width',pos_conf.pans.dims.width,'height',1.75*pos.char2norm);

					%% OK pbutton
					pos_conf.pbutOk=[pos_conf.panSub(1),pos_conf.panSub(2)-pos_conf.pans.margins.vert-pos_conf.etxts.dims.height,pos_conf.etxts.dims.width,pos_conf.etxts.dims.height];
					h_conf.pbutOk=uicontrol(h_conf.fig,'Style','pushbutton','String','OK','Units','normalized','Position',pos_conf.pbutOk,'Callback',@(hObject,eventdata) uiresume);
					uicontrol(h_conf.pbutOk);

				%% Plot lines
				guidata(h_conf.fig,struct('axMain',h_conf.ax));							% Necessary for trplGUIClasses.Line constructor
				setappdata(h_conf.fig,'meas',struct('t',[-Inf,Inf],'wl',t));			% Swap to maintain x-y definitions
				h_conf.T0Vert=trplGUIClasses.Line([t0.val,0],[t0.val,Ylims(2)],h_conf.fig,'linecolor',hsv2rgb(0,0.85,0.85),'linewidth',2,'linestyle',':','pointer','left','etxtwl1',h_conf.etxtT0);
				h_conf.SubHoriz=trplGUIClasses.Line([t(1),sub.val],[t(end),sub.val],h_conf.fig,'linecolor',hsv2rgb(0.66,0.5,0.9),'linewidth',2,'linestyle',':','pointer','top','etxtt1',h_conf.etxtSub);
				legend(h_conf.ax,'Filtered','Raw','t_0','Subtraction','Location','NorthWest');
				line('Tag','Zero','XData',[t(1),t(end)],'YData',[0,0],'Color','k','LineWidth',1,'LineStyle','-');

				%% Redefine guidata and add appdata
				guidata(h_conf.fig,h_conf);
				setappdata(h_conf.fig,'vars',struct('t0',t0,'sub',sub));

				%% Wait for user confirmation
				uiwait(h_conf.fig);

				%% Update values from appdata
				vars=getappdata(h_conf.fig,'vars');
				t0=vars.t0;
				sub=vars.sub;

				%% Close figure
				close(h_conf.fig);

			else
				[t0.val,t0.ind,sub.ind]=trplT0Find(kin,t,'draw',false,'maxfraction',opts.maxfraction,'stdevfraction',opts.stdevfraction,'sigma',opts.sigma);
				sub.val=mean(kin(1:sub.ind));
			end

			%% Translate time axis
			if(opts.t0)
				t=t-t0.val;
			end

			%% Subtraction constant
			if(opts.subtract)
				kin=kin-sub.val;
			end

			%% Normalise kinetic to t0
			if(opts.normalize)
				kin=kin./kin(t0.ind);
			end

			%% Handle crop option
			if(opts.crop)
				kin=kin(t0.ind:end);
				t=t(t0.ind:end);
			end

			%% Handle toworkspace option
			if(opts.toworkspace)

				%% Generate default variable name
				ws_var=genvarname(name);								% N.b. This just removes forbidden names. export2wsdlg handles naming conflicts.
				ws_var=genvarname(sprintf('%s_wl%.3g_%.3g',ws_var,wls{ind_file}(ind_int,:)));

				%% Open export to workspace dialog
				h_ex2ws.fig=export2wsdlg({'Time:','PL Intensity:'},{[ws_var,'_t'],[ws_var,'_pl']},{t,kin},'Export kinetic to workspace');

				%% Minor cosmetic surgery
				h_ex2ws.pbut_OK=findobj(get(h_ex2ws.fig,'Children'),'flat','String','OK');
				set(h_ex2ws.pbut_OK,'FontWeight','normal');
				uicontrol(h_ex2ws.pbut_OK);
				h_ex2ws.etxt=findobj(get(h_ex2ws.fig,'Children'),'flat','Style','edit');
				set(findjobj(h_ex2ws.etxt),'FocusGainedCallback',@trplKinTool_etxt_FocusGainedCallback);
				uiwait(h_ex2ws.fig);

			end

			%% Handle tocftool option
			if(opts.tocftool)
				var_t=genvarname(sprintf('%s_wl%.3g_%.3g_t',name,wls{ind_file}(ind_int,:)),evalin('base','who'));
				var_kin=genvarname(sprintf('%s_wl%.3g_%.3g_pl',name,wls{ind_file}(ind_int,:)),evalin('base','who'));
				assignin('base',var_t,t);											% Workaround to limitation of dynamic variables with nested functions
				assignin('base',var_kin,kin);
				evalin('base',sprintf('cftool(%s,%s);',var_t,var_kin));
				evalin('base',sprintf('clear %s %s;',var_t,var_kin));
			end

			%% Build return structure
			outstruct.(genvarname(name,who)).(genvarname(sprintf('wl%.3g_%.3g',wls{ind_file}(ind_int,:))))=struct('t',t,'pl',kin);

		end
		
	end
	
	
	
	%%-- Inline helper functions --%%
	
	%% Set logical option
	function ret=setOptionLogical(opt)
		if(isscalar(varg.values{ind_opts}) && islogical(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end

	%% Set scalar option
	function ret=setOptionFloat(opt)
		if(isscalar(varg.values{ind_opts}) && isnumeric(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end

	%% Set numerical option %%
	function ret=setOptionNum(opt)
		if(isnumeric(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end
		
end

%% t0 edit text focus callback %%
function trplKinTool_etxt_FocusGainedCallback(jhObject, jeventdata)

	set(jhObject,'selectionStart',0);
	set(jhObject,'selectionEnd',length(get(jhObject,'text')));
	
end

%% Edit text callback %%
function trplKinTool_etxt_Callback(hObject,eventdata)

	%% Get handles and appdata
	handles=guidata(gcbf);
	vars=getappdata(gcbf,'vars');
	meas=getappdata(gcbf,'meas');

	%% Get new value
	val=str2double(get(hObject,'String'));

	%% Determine etxt
	switch(get(hObject,'Tag'))
		case 'etxtT0'

			%% Assert new value
			if(isnan(val) || val>max(meas.wl) || val<min(meas.wl))
				val=vars.t0.val;
				val_ind=vars.t0.ind;
			else
				val_ind=find(meas.wl>=val,1,'first');
				val=meas.wl(val_ind);
			end

			%% Replot t0 cursor
			set(handles.T0Vert,'wl1',val,'wl2',val);
			
			%% Set t0 and t0_ind to new value
			vars.t0.val=val;
			vars.t0.ind=val_ind;

		case 'etxtSub'

			%% Assert new value
			if(isnan(val))
				val=vars.sub.val;
			end

			%% Replot subtraction cursor
			set(handles.SubHoriz,'t1',val,'t2',val);

			%% Set sub to new value
			vars.sub.val=val;

		otherwise
			throw(MException('trpl:KinTool:ETxtTag','Unexpected edit-text tag. Expected: etxtT0|etxtSub. Received: %s',get(hObject,'Tag')));
	end
	
	%% Set appdata
	setappdata(gcbf,'vars',vars);

	%% Update etxt string
	set(hObject,'String',num2str(val));

end

%% Button down callback %%
function trplKinTool_fig_WindowButtonDown_Callback(hObject,eventdata)

		%% Get handles
		handles=guidata(hObject);

		%% Return if not left-click
		if(~strcmp(get(hObject,'SelectionType'),'normal'))
			return
		end

		%% Get axes limits
		lims.pl=get(handles.ax,'YLim');
		lims.t=get(handles.ax,'XLim');

		%% Get coordinates of mouse button down
		click=get(handles.ax,'CurrentPoint');
		click=click(1,1:2);

		%% Return if mouse coordinates outside axes
		if(click(1)<lims.t(1) || click(1)>lims.t(2) || click(2)<lims.pl(1) || click(2)>lims.pl(2))
			return
		end

		%% Near point
		if(handles.T0Vert.isNear(click))
			set(hObject,'WindowButtonMotionFcn',{@trplKinTool_fig_WindowButtonMotion_Callback,'t0'});
		elseif(handles.SubHoriz.isNear(click))
			set(hObject,'WindowButtonMotionFcn',{@trplKinTool_fig_WindowButtonMotion_Callback,'sub'});
		end

end

%% Button motion callback %%
function trplKinTool_fig_WindowButtonMotion_Callback(hObject,eventdata,id)

	%% Get handles and appdata
	handles=guidata(hObject);
	vars=getappdata(hObject,'vars');
	meas=getappdata(hObject,'meas');
	
	%% Get coordinates of current mouse position
	pt=get(handles.ax,'CurrentPoint');
	pt=pt(1,1:2);
	
	switch(id)
		case 't0'
			%% Move t0 line
			vars.t0.ind=find(meas.wl>=pt(1),1,'first');
			val=meas.wl(vars.t0.ind);
			set(handles.T0Vert,'wl1',val,'wl2',val);							% Swapped wl/t for trplGUIClasses.Line
			vars.t0.val=val;
		case 'sub'
			%% Move sub line
			set(handles.SubHoriz,'t1',pt(2),'t2',pt(2));
			vars.sub.val=pt(2);
	end
	
	%% Set appdata
	setappdata(hObject,'vars',vars);

end

%% Button up callback %%
function trplKinTool_fig_WindowButtonUp_Callback(hObject, eventdata)

	%% Stop tracking mouse motion
	if(~isempty(get(hObject,'WindowButtonMotionFcn')))
		set(hObject,'WindowButtonMotionFcn',[]);
	end

end

