function kin = trplReconv(t_exp,h_sig,h_irf,span_irf)

%--	Example Usage ----------------------------------------------------------------------------------------------------------%
%																															%
%	% Command Line Usage %																									%
%																															%
% 	t=linspace(-20,140,672);											% Define time axis									%
% 	A=0.5;																% Signal function parameter 1						%
% 	tau=10;																% Signal function parameter 2						%
% 	sigma=3;															% IRF parameter 1									%
% 	h_sig=@(t,A,tau) (t>=0).*(A.*exp(-t./tau));							% Ideal signal (kinetic) function					%
% 	h_IRF=@(t,sigma) exp(-(t.^2)/(2*(sigma.^2)));						% Instrument response function (IRF)				%
% 	pl=trplReconv(t,@(t) h_sig(t,A,tau),@(t) h_IRF(t,sigma),5*sigma);	% Reconvolve										%
% 	plot(t,[h_sig(t,A,tau);h_IRF(t,sigma);pl]);							% Plot ideal function before and after convolution	%
%																															%
%																															%
%	% cftool Usage %																										%
%																															%
%	% Define a custom fit function in a file																				%
%	function pl=maFit(t,A,tau,sigma)																						%
%		h_sig=@(t,A,tau) (t>=0).*(A.*exp(-t./tau));																			%
%		h_IRF=@(t,sigma) exp(-(t.^2)/(2*(sigma.^2)));																		%
%		pl=trplReconv(t,@(t) h_sig(t,A,tau),@(t) h_IRF(t,sigma),5*sigma);													%
%	end																														%
%																															%
%	% Call function in cftool via the custom equation dialog																%
%	maFit(x,A,tau,sigma)																									%
%																															%
%---------------------------------------------------------------------------------------------------------------------------%


	%% TODO:
	%%	- Add max not found handling
	%%	- Remove span_irf argument
	%%	- Add vector IRF handling

	%% Handle t dimensions
	if(size(t_exp,2)<size(t_exp,1))
		flag_col=true;
		t_exp=t_exp';
	else
		flag_col=false;
	end
		
	%% Calculate temporal parameters
	t.exp=t_exp;
	t.lin=linspace(t.exp(1),t.exp(end),length(t.exp));
	t.step=t.lin(2)-t.lin(1);
	t.indmax=find(t.lin>=0,1,'first');
	
	%% Generate IRF
	t.irf=(-span_irf:t.step:span_irf);
	irf=h_irf(t.irf);
	
	%% Generate preliminary signal kinetic
	sig.lin=h_sig(t.lin);
	sig.norms=max(sig.lin);
	
	%% Preliminary convolution
	cv.lin=conv(sig.lin,irf);
	cv.lin(end-(length(irf)-1)+1:end)=[];													% Trim convolution tail-off

	% Possible that do not find maximum on first preliminary convolution (e.g. diff-exp with late peak) and on secondary do.
	% In this case, will not match experimental kinetics well. Check for this with n-times extension to find max. Else throw.	
	
	%% Calculate signal time axis
	[cv.max.val,cv.max.ind]=max(cv.lin);
	cv.valid=[cv.max.ind-t.indmax+1,(cv.max.ind+length(t.lin)-t.indmax)];
% 	if(cv.valid(1)<=0)																		% Never need to extend start (consider Heaviside)
% 		t.sig=t.lin(1)-(t.step.*(fliplr(1:-cv.valid(1)+1)));
% 		cv.valid=cv.valid-cv.valid(1)+1;
% 	end
	t.sig=t.lin;
	if(cv.valid(2)>length(cv.lin))
		t.sig=[t.sig,t.lin(end)+(t.step.*(1:cv.valid(2)-length(cv.lin)+length(irf)-1))];	% Extend end if necessary
	end
	
	%% Generate final signal kinetic
	sig.lin=h_sig(t.sig);
	
	%% Final convolution
	cv.lin=conv(sig.lin,irf);
	
	%% Crop and normalise
	cv.lin=cv.lin(cv.valid(1):cv.valid(2));
	cv.lin=(cv.lin./max(cv.lin)).*sig.norms;
	
	%% Interpolate to original time axis
	cv.cv=interp1(t.lin,cv.lin,t.exp);

	%% Match t dimensions and return
	kin=cv.cv;
	if(flag_col)
		kin=kin';
	end
	
end