%% Name:		trplSave.m
%% Description:	Save data from Time Resolved PL measurements performed in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function trplSave(data,wl,t,stat,varargin)

	%% Syntax:
	%%
	%%		trplSave(data,wl,t,stat,'path')
	%%
	%% Description:
	%%
	%%		[data,wl,t,stat] = trplSave(data,wl,t,stat,'path') saves TRPL data to the file designated by 'path'. Currently, two output
	%%		file formats are supported; Hamamatsu HPD-TA ITEX files (*.img), and calibrated ASCII files (*.dac). The appropriate file format
	%%		is automagically detected from the file extension.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				wl:		Column vector defining the horizontal wavelength axis with wl(i) corresponding to data(:,i).	
	%%				t:		Column vector defining the vertical time axis with t(i) corresponding to data(i,:).
	%%				stat:	Structure containing information held in status string in the same format as returned by trplLoad.
	%%				'path':	String designating file path
	%%
	%%			Returns:
	%%				None

	%% GUI path selection (if no path argument or directory)
	exts={'*.img','ITEX image (*.img)';'*.dac','Calibrated ASCII image (*.dac)';'*.*','All Files (*.*)'};
	if(isempty(varargin))
		[file_name,file_path]=uiputfile(exts,'Save TRPL data');
		path=[file_path,file_name];
	elseif(isdir(varargin{1}))
		[file_name,file_path]=uiputfile(exts,'Save TRPL data',varargin{1});
		path=[file_path,file_name];
	else
		path=varargin{1};
	end
	
	%% Catch cancel from uiputfile
	if(~any(path))
		return
	end
	
	%% Detect file format based on extension
	[file_path,file_name,file_ext]=fileparts(path);
	
	%% Save data according to format
	switch(file_ext)
		case '.img'
			trplSave_ITEX(data,wl,t,stat,path);
		case '.dac'
			trplSave_ASCIICal(data,wl,t,stat,path);
		otherwise
			throw(MException('trpl:Save:BadFormat','File format *%s not yet supported',file_ext));
	end

end

function trplSave_ASCIICal(data,wl,t,stat,path)

	%% Check status structure fields
	if(~(isfield(stat,'Scaling') && all(isfield(stat.Scaling,{'ScalingXUnit';'ScalingYUnit'}))))
		warning('trpl:SaveASCII:Stat','Status structure incomplete. Cannot determine units. Continuing with unknown units');
		Scaling.ScalingXUnit='?';
		Scaling.ScalingYUnit='?';
		stat.Scaling=Scaling;
	end

	%% Open file
	fid=fopen(path,'wt');
	if(fid<0)
		throw(MException('trpl:SaveASCII:Open','Could not open file %s',path));
	end
	
	%% Write header line
	fprintf(fid,'%s\t',[stat.Scaling.ScalingYUnit,'|',stat.Scaling.ScalingXUnit]);
	
	%% Close file
	fclose(fid);
	
	%% Determine newline
	if(ispc())
		newline='pc';
	else
		newline='unix';
	end
	
	%% Write data
	dlmwrite(path,wl','-append','delimiter','\t','newline',newline);
	dlmwrite(path,[t,data],'-append','delimiter','\t','newline',newline);

end

function trplSave_ITEX(data,wl,t,stat,path)
	
	%% Determine datatype
	datatype=struct('string','','bytes',[],'numerical',[]);
	if(isfield(stat.Acquisition,'DataTypeString'))
		
		%% XITEX
		datatype=trplSave_DataType(stat.Acquisition.DataTypeString);
		if(isempty(datatype))
			warning('trpl:SaveITEX:UnkownDataType','Unexpected storage datatype encountered in status structure (Acquisition->DataTypeString). Expected uint8, int16, int32, single or double. Received: %s. Defaulting to actual data class (%s) as storage datatype',stat.Acquisition.DataTypeString,class(data));
			datatype=trplSave_DataType(class(data));
			stat.Acquisition.DataTypeString=datatype.string;
		end				
		if(~strcmp(stat.Acquisition.DataTypeString,class(data)))		% Could techincally fall over here if isempty(datatype)
			warning('trpl:SaveITEX:DataTypeClash','Incompatible data types found in status structure and data class. Value from status structure (Acquisition->DataTypeString): %s. Actual data class: %s. Modifying status structure value to match data class',stat.Acquisition.DataTypeString,class(data));
			datatype=trplSave_DataType(class(data));
			stat.Acquisition.DataTypeString=datatype.string;
		end
			
	else
		
		%% ITEX	
		datatype.bytes=uint8(str2double(stat.Acquisition.BytesPerPixel));
		datatype.string=sprintf('int%d',datatype.bytes*8);
		if(datatype.bytes==1)
			datatype.string=['u',datatype.string];
		end
		datatype=trplSave_DataType(datatype.string);
		if(isempty(datatype))
			warning('trpl:SaveITEX:UnkownIntType','Unexpected integer storage datatype encountered in status structure (Acquisition->BytesPerPixel). Expected 1 (uint8), 2 (int16) or 4 (int32). Received: %s. Defaulting to actual data class (%s) as storage datatype',stat.Acquisition.BytesPerPixel,class(data));
			datatype=trplSave_DataType(class(data));
			stat.Acquisition.BytesPerPixel=sprintf('%d',datatype.bytes);
		end	
		if(~strcmp(datatype.string,class(data)))		% Could techincally fall over here if isempty(datatype)
			warning('trpl:SaveITEX:IntTypeClash','Incompatible integer data types found in status structure and data class. Value from status structure (Acquisition->BytesPerPixel): %s. Actual data class: %s. Modifying status structure value to match data class',datatype.string,class(data));
			datatype=trplSave_DataType(class(data));
			stat.Acquisition.BytesPerPixel=sprintf('%d',datatype.bytes);
		end
		
	end
	if(isempty(datatype))
		throw(MException('trpl:SaveITEX:BadDataType','Unsupported storage datatype encountered. Expected uint8, int16, int32, single or double. Received: %s',class(data)));
	end
	
	%% Linear scaling
	tabmask=[uint8(str2double(stat.Scaling.ScalingXType))==1,uint8(str2double(stat.Scaling.ScalingYType))==1];
	if(any(tabmask))
		
		%% X-scaling
		if(tabmask(1))
			
			%% Assert scaling offset
			if(isfield(stat.Scaling,'ScalingXOffset') && ~isempty(stat.Scaling.ScalingXOffset) && (str2double(stat.Scaling.ScalingXOffset)~=double(wl(1))))
				warning('trpl:SaveITEX:ScalingXOffsetClash','Incompatible X axis scaling offsets found in status structure and axis data. Value from status structure: %s. Value from axis data: %g. Modifying status structure value to match axis data',stat.Scaling.ScalingXOffset,wl(1));
				stat.Scaling.ScalingXOffset=sprintf('%f',wl(1));
			end
			
		end
		
		%% Y-scaling
		if(tabmask(2))
			
			%% Assert scaling offset
			if(isfield(stat.Scaling,'ScalingYOffset') && ~isempty(stat.Scaling.ScalingYOffset) && (str2double(stat.Scaling.ScalingYOffset)~=double(t(1))))
				warning('trpl:SaveITEX:ScalingYOffsetClash','Incompatible Y axis scaling offsets found in status structure and axis data. Value from status structure: %s. Value from axis data: %g. Modifying status structure value to match axis data',stat.Scaling.ScalingYOffset,t(1));
				stat.Scaling.ScalingYOffset=sprintf('%f',t(1));
			end
			
		end
		
	end
	
	%% Tabular scaling
	%
	%	- Might need to check for addressing overflow for #-type indicators
	%		- Would cause issues with HPD-TA though
	%		- Not necessary for Chameleon as 512*672 ~ 1E6
	%		- Might be an issue for TA/L16 data though
	%		- L16 should be linearly scaled though so only an issue for TA
	%	- Alternative addressing formats (+,*) not important for the moment
	%
	tabmask=[uint8(str2double(stat.Scaling.ScalingXType))==2,uint8(str2double(stat.Scaling.ScalingYType))==2];
	if(any(tabmask))
		
		params=struct('hashindfmt','#%07u,%04u','bytes',struct('stat',length(struct2ini(stat)),'header',64,'scaling',4));
		scale=struct('x',struct('address',0,'entries',0),'y',struct('address',0,'entries',0));
		
		%% X-scaling
		if(tabmask(1))
			scale.x.address=params.bytes.header+params.bytes.stat+(datatype.bytes*numel(data));
			scale.x.entries=length(wl);
			stat.Scaling.ScalingXScalingFile=sprintf(params.hashindfmt,scale.x.address,scale.x.entries);
		end
		
		%% Y-scaling
		if(tabmask(2))
			scale.y.address=params.bytes.header+params.bytes.stat+(datatype.bytes*numel(data))+(params.bytes.scaling*scale.x.entries);
			scale.y.entries=length(t);
			stat.Scaling.ScalingYScalingFile=sprintf(params.hashindfmt,scale.y.address,scale.y.entries);
		end
		
	end
	
	%% Convert status structure
	stat_str=struct2ini(stat);
	
	%% Open file
	fid=fopen(path,'w');
	if(fid<0)
		throw(MException('trpl:SaveITEX:Open','Error opening file %s for writing',path));
	end

	%% Write magic number
	fwrite(fid,'IM','uint8');

	%% Write status string length [bytes]
	fwrite(fid,length(stat_str),'uint16');

	%% Write image width and height [pixels]
	fwrite(fid,size(data,2),'uint16');
	fwrite(fid,size(data,1),'uint16');

	%% Write X and Y offsets [pixels]
	%
	%	- Loading using trplLoad loses offset info as axes have already been cropped
	%	- Therefore always write zero offset here
	%	- This may cause compatibility issues with HPD-TA for data which have been JCed
	%
	fwrite(fid,0,'uint16');
	fwrite(fid,0,'uint16');

	%% Write data type
	fwrite(fid,datatype.numerical,'uint16');

	%% Write reserved bytes
	fwrite(fid,zeros(1,50),'uint8');

	%% Write status string
	fwrite(fid,stat_str,'uint8');

	%% Write data
	fwrite(fid,data',datatype.string);

	%% Write scaling data
	if(tabmask(1))
		fwrite(fid,wl,'float32');
	end
	if(tabmask(2))
		fwrite(fid,t,'float32');
	end
	
	%% Close file
	fclose(fid);

end

function type = trplSave_DataType(str)

	switch(str)
		case 'uint8'
			type.string='uint8';
			type.bytes=1;
			type.numerical=0;
		case 'int16'
			type.string='int16';
			type.bytes=2;
			type.numerical=2;
		case 'int32'
			type.string='int32';
			type.bytes=4;
			type.numerical=3;
		case 'single'
			type.string='single';
			type.bytes=4;
			type.numerical=4;
		case 'double'
			type.string='double';
			type.bytes=8;
			type.numerical=5;
		otherwise
			type=[];
	end

end

